#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Author: wuyue
# Email: wuyue92tree@163.com


from __future__ import print_function, unicode_literals

import email
import re
import traceback
import imaplib

from imapclient import IMAPClient
from email.header import decode_header

imaplib._MAXLINE = 10000000

SEEN = br'\Seen'


class MailReceiver(IMAPClient):
    def __init__(self, host, timeout=60, **kwargs):
        super(MailReceiver, self).__init__(host, timeout=timeout, **kwargs)

    def get_folder_list(self):
        """
        获取邮箱文件夹
        :return: list
        """
        folders = self.list_folders()
        res_list = []
        for folder in folders:
            if folder:
                # print(folder[2].encode("utf-8"))
                res_list.append(folder[2])
        return res_list

    def get_message_id_list(self, mailbox='INBOX', search_='all'):
        """
        获取邮件ID列表
        :param mailbox: 邮箱文件夹
        :param search_: 搜索规则
        :return: list
        """
        self.select_folder(mailbox)
        # message_list = self.server.search('ON 21-Dec-2017')
        message_list = self.search(search_)
        return message_list

    def get_message_list(self, message_id_list):
        """
        获取邮件列表
        :param message_id_list: 邮件ID列表
        :return: dict   id:email
        """
        message_list = self.fetch(
            message_id_list, ['INTERNALDATE', 'FLAGS', 'BODY.PEEK[]'])
        if not message_list:
            return

        return message_list

    @staticmethod
    def parse_email(m, flag=None):
        """
        解析邮件header内容
        :param m: 原内容
        :param flag: 解析类型标识
        :return: 编码转换后内容
        """
        res = []
        try:
            for s, c in decode_header(m):
                if c:
                    res.append(s.decode(c, 'ignore'))
                else:
                    res.append(s.decode('utf-8') if isinstance(s, bytes) else s)

            if not res:
                return

            # 处理邮件发送方 返回邮箱地址
            if flag == 'from':
                res = re.findall(
                    '[0-9a-zA-Z_\.]{0,19}@[0-9a-zA-Z\.]{1,100}', res[1])
                return res[0]

            # 处理邮件接收方 返回邮箱地址列表
            if flag == 'to':
                new_res = []
                for e in res[0].split(','):
                    em = re.findall('[0-9a-zA-Z_\.]{0,19}@[0-9a-zA-Z\.]{1,100}', e)
                    if em:
                        new_res.append(em[0])

                return new_res

            return res[0]
        except Exception as e:
            traceback.print_exc()
            return res

    def get_message_content(self, message):
        """
        获取邮件内容
        :param message:
        :return:
        """
        try:
            while True:
                res = {}
                msg = email.message_from_bytes(message[b'BODY[]'])
                res['subject'] = self.parse_email(msg['Subject'])
                res['from'] = self.parse_email(msg['From'], flag='from')
                res['to'] = self.parse_email(msg['To'], flag='to')
                res['date'] = self.parse_email(msg['Date'])

                for par in msg.walk():
                    if not par.is_multipart():
                        name = par.get_param("name")
                        if name:
                            # print(name)
                            pass
                        else:
                            body = par.get_payload(decode=True)
                            if not body:
                                continue
                            try:
                                code = par.get_content_charset()
                                res['body'] = body.decode(code, 'ignore')
                            except TypeError:
                                res['body'] = body
                return res

        except Exception as e:
            traceback.print_exc()
            return

    def delete_message(self, messages, deleted_folder="Deleted Messages"):
        """
        删除邮件
        :param messages:
        :param deleted_folder:
        :return:
        """
        try:
            self.add_flags(messages, SEEN)
            if deleted_folder:
                # 将邮件移动到 已删除
                self.copy(messages, deleted_folder)
            self.delete_messages(messages)
            self.expunge()
            return True
        except Exception as e:
            traceback.print_exc(e)
            return False



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: middlewares.py
@create at: 2018-06-26 18:21

这一行开始写关于本文件的说明与解释

通过redis hash表记录站点cookie

key为 cookie_pool:SITE (SITE需要在settings中指定)
field为 账号cookie的唯一标识，可以是username，id等，具体自行约定
value为 cookie值，必须为json格式

"""
import datetime
import random
from scrapy.downloadermiddlewares.retry import RetryMiddleware
from scrapy.exceptions import IgnoreRequest
from scrapy_redis.connection import get_redis_from_settings

from crwy.utils.common import datetime2str
from crwy.utils.data.RedisHash import RedisHash
from crwy.exceptions import CrwyScrapyPlugsException, CrwyCookieValidException
from crwy.utils.scrapy_plugs.dupefilters import release_dupefilter_key


class CookieMiddleware(RetryMiddleware):
    """
    cookie_pool
        eg: '{"a": 1, "b": "aaa"}'
    """

    def __init__(self, settings):
        super(CookieMiddleware, self).__init__(settings)
        self.site = settings.get('SITE', None)
        if not self.site:
            raise CrwyScrapyPlugsException('SITE_NOT_SET')

        self.server = get_redis_from_settings(settings)

        self.h = RedisHash(
            'cookie_pool:{}'.format(self.site),
            server=self.server
        )

    def process_request(self, request, spider):
        """
        cookie_user不为空时，获取cookie_user对应的cookie
        keep_cookie_user为True时，将设置cookie_user，并传递到子请求
        :param request:
        :param spider:
        :return:
        """
        if request.meta.get('cookie_user', ''):
            user = request.meta.get('cookie_user')
        else:
            users = self.h.hkeys()
            if len(users) > 0:
                user = random.choice(users)
                if request.meta.get('keep_cookie_user', False) is True:
                    request.meta['cookie_user'] = user
            else:
                raise CrwyScrapyPlugsException(
                    'no user in cookie_pool:{}'.format(self.site))
        cookie = self.h.hget(user)
        if cookie:
            # 字典存入redis，取出时未string，通过eval进行还原
            request.cookies = eval(cookie)
            spider.logger.debug('get_cookie_success: {}'.format(
                user.decode('utf-8')))
        else:
            spider.logger.warning('get_cookie_failed: {}'.format(
                user.decode('utf-8')))


class LimitCookieMiddleware(CookieMiddleware):
    """
    在cookieMiddleware基础上限制账号

    1. 每日搜索上限
        通过 update_requests_count method 更新有效请求次数
    2. cookie失效标识
        捕捉 CrwyCookieValidException 异常更改标识状态
    """
    def __init__(self, settings):
        super(LimitCookieMiddleware, self).__init__(settings)

        # 每日搜索上限
        self.search_limit = RedisHash(
            'search_limit:{}'.format(self.site), server=self.server)
        # cookie失效标识， 1为cookie失效
        self.account_limit = RedisHash(
            'account_limit:{}'.format(self.site), server=self.server)

    def get_requests_count(self, request):
        user = request.meta.get('cookie_user')
        today = datetime2str(datetime.datetime.now(), fmt='%Y-%m-%d')
        if not self.search_limit.hget(user):
            count = 1
        else:
            date, count = self.search_limit.hget(
                user).decode('utf-8').split('|')
            if date == today:
                count = int(count)
            else:
                count = 1
        return user, count

    def update_requests_count(self, request, spider):
        """
        请求完毕后添加详情页计数
        :param request:
        :param spider:
        :return:
        """
        user, count = self.get_requests_count(request)
        today = datetime2str(datetime.datetime.now(), fmt='%Y-%m-%d')
        count += 1
        self.search_limit.hset(user, '{}|{}'.format(today, count))
        spider.logger.debug('update search_limit: {} {}'.format(
            user.decode('utf-8'), count))

    def _retry(self, request, reason, spider):
        callback = super(LimitCookieMiddleware, self)._retry(
            request, reason, spider
        )
        if not callback:
            if isinstance(reason, CrwyCookieValidException):
                user = request.meta.get('cookie_user')
                self.account_limit.hset(user, 1)
                spider.logger.warning('account limit: {} after retry'.format(
                    user.decode('utf-8')))
                raise IgnoreRequest
        else:
            return callback

    def process_request(self, request, spider):
        super(LimitCookieMiddleware, self).process_request(request, spider)

        user, count = self.get_requests_count(request)

        dupefilter_key = request.meta.get('dupefilter_key')

        # 判断account_limit, 若为1则表示账号禁用中
        if self.account_limit.hget(user) == b'1':
            spider.logger.warning(
                'account_limit: {}'.format(user.decode('utf-8')))
            release_dupefilter_key.call(spider, dupefilter_key)
            raise IgnoreRequest

        # 判断是否为受保护搜索账号
        if user.decode('utf-8') in spider.custom_settings.get(
                'SAFE_SEARCH_ACCOUNT'):
            if count >= spider.custom_settings.get('SAFE_SEARCH_LIMIT'):
                spider.logger.warning(
                    '{} safe_search_limit: {}'.format(
                        user.decode('utf-8'), count))
                release_dupefilter_key.call(spider, dupefilter_key)
                raise IgnoreRequest

        # 判断search_limit，若大于上限则跳过
        if count >= spider.custom_settings.get('SEARCH_LIMIT'):
            spider.logger.warning(
                '{} search_limit: {}'.format(user.decode('utf-8'), count))
            release_dupefilter_key.call(spider, dupefilter_key)
            raise IgnoreRequest

        if not request.cookies:
            spider.logger.warning('cookie is empty: {}'.format(
                user.decode('utf-8')))
            release_dupefilter_key.call(spider, dupefilter_key)
            raise IgnoreRequest



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: IntelliJ IDEA
@file: selenium_api.py
@create at: 2018-10-15 11:51

这一行开始写关于本文件的说明与解释
"""

import os
import re
import time
import uuid

from PIL import Image
from crwy.spider import Spider
from crwy.exceptions import CrwyImportException

try:
    from selenium import webdriver
except ImportError:
    raise CrwyImportException(
        "You should install selenium first! suggestion: pip install "
        "selenium==3.6.0")
from selenium.common.exceptions import TimeoutException, NoSuchElementException
from selenium.webdriver import DesiredCapabilities, ActionChains
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait


class SeleniumApi(Spider):
    def __init__(self, driver_type='chrome',
                 hub_url='http://127.0.0.1:4444/wd/hub',
                 proxy=None, user_agent=None, use_hub=True,
                 logger=None):
        super(SeleniumApi, self).__init__(logger=logger)
        """
        :param driver_type: driver类型
        :param hub_url: hub server地址
        :param proxy: 代理地址
        :param user_agent: user_agent
        :param use_hub: 是否启用hub，为False时使用本地driver
        """
        self.driver_type = driver_type
        self.hub_url = hub_url
        self.proxy = proxy
        self.use_hub = use_hub
        self.user_agent = user_agent
        # device_pixel_ratio 用于处理高分屏dpi抠图
        self.device_pixel_ratio = 1
        self.driver = self.init_driver()
        self.driver.set_window_size(1280, 960)

    def _init_chrome_driver(self):
        chrome_options = webdriver.ChromeOptions()
        if self.proxy:
            chrome_options.add_argument('--proxy-server=%s' % self.proxy)
        if self.user_agent:
            chrome_options.add_argument('--user-agent=%s' % self.user_agent)
        desired_capabilities = chrome_options.to_capabilities()
        if self.use_hub:
            driver = webdriver.Remote(
                command_executor=self.hub_url,
                desired_capabilities=desired_capabilities
            )
        else:
            driver = webdriver.Chrome(
                chrome_options=chrome_options
            )
        return driver

    def _init_firefox_driver(self):
        firefox_profile = webdriver.FirefoxProfile()
        if self.proxy:
            ip, port = self.proxy.split(':')
            firefox_profile.set_preference('network.proxy.type', 1)
            firefox_profile.set_preference('network.proxy.http', ip)
            firefox_profile.set_preference('network.proxy.http_port', int(port))
            firefox_profile.set_preference('network.proxy.ssl', ip)
            firefox_profile.set_preference('network.proxy.ssl_port', int(port))
        if self.user_agent:
            firefox_profile.set_preference(
                'general.useragent.override', self.user_agent)
        firefox_profile.update_preferences()
        desired_capabilities = DesiredCapabilities.FIREFOX
        if self.use_hub:
            driver = webdriver.Remote(
                command_executor=self.hub_url,
                desired_capabilities=desired_capabilities,
                browser_profile=firefox_profile
            )
        else:
            driver = webdriver.Firefox(
                firefox_profile=firefox_profile,
                capabilities=desired_capabilities
            )
        return driver

    def init_driver(self):
        if self.driver_type.upper() == 'CHROME':
            return self._init_chrome_driver()
        elif self.driver_type.upper() == 'FIREFOX':
            return self._init_firefox_driver()
        raise Exception('No supported driver: %s' % self.driver_type)

    @staticmethod
    def is_similar(image1, image2, x, y, distance=25):
        """
        对比RGB值
        :param image1: 待对比的图片1
        :param image2: 待对比的图片2
        :param x: x坐标
        :param y: y坐标
        :param distance: 色差
        :return:
        """
        # 获取指定位置的RGB值
        pixel1 = image1.getpixel((x, y))
        pixel2 = image2.getpixel((x, y))
        for i in range(0, 3):
            # 如果相差超过50则就认为找到了缺口的位置
            # print(x, y, pixel1, pixel2)
            if abs(pixel1[i] - pixel2[i]) >= distance:
                return False
        return True

    def get_diff_location(self, image1, image2):
        """
        计算缺口的位置
        :param image1:
        :param image2:
        :return:
        """
        i = 0
        # 两张原始图的大小都是相同的260*160
        # 那就通过两个for循环依次对比每个像素点的RGB值
        # 如果相差超过50则就认为找到了缺口的位置
        for i in range(0, image1.width):
            for j in range(0, image1.height):
                if self.is_similar(image1, image2, i, j) is False:
                    return i
        return i

    def get_img(self, screenshot, xpath):
        """
        获取验证码图片

        :param screenshot: 页面截图
        :param xpath: 验证码图片xpath
        :return: img对象
        """
        self.device_pixel_ratio = self.driver.execute_script(
            "return window.devicePixelRatio;")
        element = self.driver.find_element_by_xpath(xpath)
        left = int(element.location['x']) * self.device_pixel_ratio
        top = int(element.location['y']) * self.device_pixel_ratio
        right = int(element.location['x'] +
                    element.size['width']) * self.device_pixel_ratio
        bottom = int(element.location['y'] +
                     element.size['height']) * self.device_pixel_ratio
        img = Image.open(screenshot)
        img = img.crop((left, top, right, bottom))
        return img

    def click_img(self, answer, height, identify_img_xpath1=None,
                  identify_button_xpath=None):
        """
        根据打码返回的坐标进行点击操作
        仅适用与点击型验证码

        :param answer: 打码返回结果
        :param height: 答案高度
        :param identify_img_xpath1: 题目xpath
        :param identify_button_xpath: 验证按钮
        :return:
        """
        actions = ActionChains(self.driver)
        img = self.driver.find_element_by_xpath(identify_img_xpath1)
        points = answer.split('|')
        for point in points:
            x, y = eval(point)
            actions.move_to_element_with_offset(
                img, x, y - int((height / self.device_pixel_ratio)))
            actions.click()
        actions.perform()
        time.sleep(2)
        if not identify_button_xpath:
            return
        self.driver.find_element_by_xpath(identify_button_xpath).click()

    def deal_normal_verification_code(
            self, captcha_obj, captcha_code, identify_img_xpath):
        uuid_str = str(uuid.uuid1())
        screenshot_path = './data/img/screenshot_%s.png' % uuid_str
        check_img_path = './data/img/check_image-%s.png' % uuid_str
        self.driver.save_screenshot(screenshot_path)
        img = self.get_img(screenshot_path, xpath=identify_img_xpath)
        img.save(check_img_path)
        answer = captcha_obj.decode(check_img_path, captcha_code)
        self.logger.info('get normal captcha code : %s' % answer)
        os.remove(screenshot_path)
        os.remove(check_img_path)
        return answer

    def deal_click_verification_code(
            self, captcha_obj, captcha_code,
            identify_img_xpath, identify_img_xpath1,
            identify_button_xpath):
        """
        处理点击型验证码

        :param captcha_obj:
        :param captcha_code:
        :param identify_img_xpath:
        :param identify_img_xpath1:
        :param identify_button_xpath:
        :return:
        """
        uuid_str = str(uuid.uuid1())
        screenshot_path = './data/img/screenshot_%s.png' % uuid_str
        check_img_path = './data/img/check_image-%s.png' % uuid_str
        self.driver.save_screenshot(screenshot_path)
        img1 = self.get_img(screenshot_path, xpath=identify_img_xpath)
        img2 = self.get_img(screenshot_path, xpath=identify_img_xpath1)
        to_image = Image.new('RGBA', (img2.width, img1.height + img2.height))
        to_image.paste(img1, (0, 0))
        to_image.paste(img2, (0, img1.height))
        if self.device_pixel_ratio > 1:
            to_image = to_image.resize(
                (int(to_image.width / self.device_pixel_ratio),
                 int(to_image.height / self.device_pixel_ratio))
            )
        to_image.save(check_img_path)
        answer = captcha_obj.decode(check_img_path, captcha_code)
        self.logger.info('get click captcha code : %s' % answer)
        self.click_img(answer, img1.height,
                       identify_img_xpath1=identify_img_xpath1,
                       identify_button_xpath=identify_button_xpath)
        os.remove(screenshot_path)
        os.remove(check_img_path)

    def get_mobile_code(self, phone_obj, phone, phone_token,
                        check_str='智联招聘', regexp='\d+',
                        retry_times=20, sleep_time=5):
        while retry_times > 0:
            msg = phone_obj.get_message(token=phone_token, phone=phone)
            if check_str in msg:
                code = re.findall(regexp, msg)[0]
                self.logger.info(
                    '{}: get mobile code success. code is: {}'.format(
                        phone, code))
                return code

            self.logger.info(
                '{}: no more message received. sleep {}s'.format(
                    phone, sleep_time))
            time.sleep(sleep_time)
            retry_times -= 1

    def is_element_visible(self, element):
        """
        判断元素是否存在
        :param element:
        :return:
        """
        driver = self.driver
        try:
            the_element = EC.visibility_of_element_located(element)
            assert the_element(driver)
            flag = True
        except (AssertionError, NoSuchElementException):
            self.logger.warning('the element is not visible.')
            flag = False
        except Exception as e:
            self.logger.exception(e)
            flag = False
        return flag

    def wait_element(self, by, by_value, timeout=5):
        try:
            WebDriverWait(
                self.driver, timeout).until(
                EC.presence_of_element_located((by, by_value)))
        except TimeoutException as e:
            self.logger.exception(e)

    @staticmethod
    def cookies2dict(cookies):
        """
        trans cookies
        :param cookies: driver.get_cookies()
        :return:
        """
        cookie_dict = {}
        for item in cookies:
            cookie_dict[item['name']] = item['value']
        return cookie_dict

    def release(self):
        try:
            self.driver.quit()
            self.driver.close()
        except:
            pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: IntelliJ IDEA
@file: xunma.py
@create at: 2018-09-14 11:41

这一行开始写关于本文件的说明与解释
"""


from __future__ import print_function, unicode_literals

from crwy.spider import Spider
from crwy.exceptions import CrwyExtendException


class XunMa(Spider):
    def __init__(self, username, password, item_id):
        super(XunMa, self).__init__()
        if username and password and item_id:
            self.username = username
            self.password = password
            self.item_id = item_id
        else:
            raise CrwyExtendException("[XunMa] params not valid.")

    def login(self):
        """
        XunMa 登录
        :return: 登录token
        """
        try:
            url = "http://xapi.xunma.net/Login?uName={username}" \
                  "&pWord={password}&Code=UTF8".format(username=self.username,
                                                       password=self.password)
            res = self.html_downloader.download(url)

            return res.text.strip().split("&")[0]
        except Exception as e:
            raise CrwyExtendException(e)

    def get_phone(self, token, phone_type='', phone=''):
        """
        获取手机号
        :param token:   登录token
        :param phone_type:  运营商 1 [移动] 2 [联通] 3 [电信]
        :param phone:   指定号码
        :return: 手机号码
        """
        try:
            url = "http://xapi.xunma.net/getPhone?ItemId=" \
                  "{item_id}&token={token}&" \
                  "PhoneType={phone_type}&Code=UTF8&" \
                  "Phone={phone}".format(token=token, item_id=self.item_id,
                                         phone_type=phone_type, phone=phone)

            res = self.html_downloader.download(url)
            return res.text.strip().split(';')[0]

        except Exception as e:
            raise CrwyExtendException(e)

    def get_message(self, token, phone):
        """
        获取短信消息
        :param token:   登录token
        :param phone:   手机号
        :return:
        """
        try:
            # http://xapi.xunma.net/getMessage?token=登陆token&itemId=项目ID&phone=手机号码
            url = "http://xapi.xunma.net/getMessage?" \
                  "token={token}&itemId={item_id}&phone={phone}" \
                  "&Code=UTF8".format(token=token,
                                      item_id=self.item_id, phone=phone)
            res = self.html_downloader.download(url)

            return res.text.strip().split('&')[-1]

        except Exception as e:
            raise CrwyExtendException(e)

    def release_phone(self, token, phone):
        try:
            # http://xapi.xunma.net/releasePhone?token=登陆token&phoneList=phone-itemId;phone-itemId;
            url = "http://xapi.xunma.net/releasePhone?" \
                  "token={token}&phoneList={phone};" \
                  "&Code=UTF8".format(token=token, phone=phone)
            self.html_downloader.download(url)

        except Exception as e:
            raise CrwyExtendException(e)

    def add_black(self, token, phone):
        try:
            url = "http://xapi.xunma.net/addBlack?" \
                  "token={token}&phoneList={phone};" \
                  "&Code=UTF8".format(token=token, phone=phone)
            self.html_downloader.download(url)

        except Exception as e:
            raise CrwyExtendException(e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

from __future__ import print_function
import sys
import os
import shutil

from crwy.settings.default_settings import TEMPLATE_DIR
from crwy.changetmpl import change_project_name

PATH = os.path.join(TEMPLATE_DIR, 'project')
DATA_PATH = os.path.join(PATH, 'data')
SRC_PATH = os.path.join(PATH, 'src')
LOG_PATH = os.path.join(PATH, 'log')
CONFIG_PATH = os.path.join(PATH, 'crwy.cfg.tmpl')
SETTINGS_PATH = os.path.join(PATH, 'settings.py.tmpl')

if sys.version_info[0] == 2:
    LOGCONFIG_PATH = os.path.join(PATH, 'logger_py2.conf.tmpl')
else:
    LOGCONFIG_PATH = os.path.join(PATH, 'logger_py3.conf.tmpl')


class Command(object):
    def create_project(self, project_name):
        os.mkdir(project_name)
        os.mkdir(project_name + '/conf')
        shutil.copytree(DATA_PATH, project_name + '/' + 'data')
        shutil.copytree(SRC_PATH, project_name + '/' + 'src')
        shutil.copytree(LOG_PATH, project_name + '/' + 'log')
        shutil.copy(LOGCONFIG_PATH, project_name + '/conf/logger.conf')

        config = change_project_name(project_name, CONFIG_PATH)
        f1 = open(project_name + '/crwy.cfg', 'w')
        f1.write(config)

        settings = change_project_name(project_name, SETTINGS_PATH)
        f2 = open(project_name + '/conf/settings.py', 'w')
        f2.write(settings)

        f3 = open(project_name + '/conf/__init__.py', 'w')
        f3.write('')

    def main(self):
        Usage = "Usage:  crwy startproject <project_name>\n"

        try:
            project_name = sys.argv[2]
            if project_name == '-h':
                print(Usage)
                print("  Add a new project")
                sys.exit(1)

            if os.path.exists(project_name):
                print('[ERROR] Path "%s" has exists!!!' % project_name)
            else:

                self.create_project(project_name)

                print("Project start......enjoy^.^")
        except IndexError:
            print(Usage)
            print("[ERROR] Please enter a project_name!!!")



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: PyCharm
@file: redis_m.py
@create at: 2017-12-26 14:50

这一行开始写关于本文件的说明与解释
"""

from crwy.exceptions import CrwyImportException
from crwy.decorates import cls2singleton

try:
    import redis
except ImportError:
    raise CrwyImportException(
        "You should install redis plugin first! try: pip install redis==2.10.6")


@cls2singleton
class RedisDb(object):
    def __init__(self, **kwargs):
        if 'url' in kwargs.keys():
            url = kwargs.pop('url')
            self.pool = redis.ConnectionPool.from_url(url, **kwargs)
        else:
            self.pool = redis.ConnectionPool(**kwargs)
        self.db = redis.StrictRedis(connection_pool=self.pool)


def get_redis_client(**kwargs):
    r = RedisDb(**kwargs)
    return r.db



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

DATEBASE_DIR = os.path.join(BASE_DIR, 'data')

TEMPLATE_DIR = os.path.join(BASE_DIR, 'templates')

CONF_DIR = os.path.join(BASE_DIR, 'crwy')


#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: IntelliJ IDEA
@file: pyppeteer_api.py
@create at: 2019-03-24 17:04

这一行开始写关于本文件的说明与解释
"""

import asyncio
from crwy.spider import Spider

try:
    from pyppeteer import launch
except ImportError:
    pass


class PyppeteerApi(Spider):
    def __init__(self, logger=None, proxy=None, **kwargs):
        super(PyppeteerApi, self).__init__(logger=logger)


def main():
    executor = PyppeteerApi()
    # TODO


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: wuyue92tree@163.com

from string import Template

try:
    import ConfigParser
except ImportError:
    from configparser import ConfigParser


def get_project_name():
    conf = ConfigParser()
    conf.read('crwy.cfg', encoding='utf-8')
    project_name = conf.get('project', 'project_name').encode('utf-8')
    return project_name


def change_project_name(name, path):
    f = open(path, 'r')
    t = Template(f.read()).substitute(project_name=name)
    return t


def change_spider_name(name, path):
    f = open(path, 'r')
    class_name = name.capitalize()
    spider_name = name
    project_name = get_project_name()
    t = Template(f.read()).substitute(class_name=class_name, spider_name=spider_name, project_name=project_name)
    return t



#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: wuyue
@contact: wuyue92tree@163.com
@software: IntelliJ IDEA
@file: font_analysis.py
@create at: 2018-08-22 19:42

这一行开始写关于本文件的说明与解释

本工具类适用于58同城，其他站点可将该类作为基础类进行扩展

思路：

1. 获取web页面内容；
2. 获取字体文件；
3. 获取字体xml文件，解析出经过自定义的文字；
4. 获取文字value的hash值（经测试发现，文字的key、value对应关系每次请求都是变化的，而不可能
改变的是字的value，所以这里通过文字value的hash值来确定，是哪一个字，反推确定页面上的字符对应的
字是什么。）

like：

'77880914931fb6dda97269a9156404745f609d35': '黄'

hash值与文字对应mapping需要人工，通过字体软件对应 推荐 fontforge，或百度字体工具在线编辑 http://fontstore.baidu.com/static/editor/index.html

5. 通过人工确认的mapping，找到页面上字符与真实字体的对应关系；
6. 替换原始页面中的字符

"""

import base64
import hashlib
import json
import os
import re
import uuid
from fontTools.ttLib import TTFont
from crwy.spider import Spider


class FontAnalysis(Spider):
    def __init__(self, html=None, font_path=None, xml_path=None):
        super(FontAnalysis, self).__init__()
        uid = str(uuid.uuid1())
        self.font_path = font_path \
            if font_path else './data/font/font-{}.woff'.format(uid)
        self.xml_path = xml_path \
            if xml_path else './data/xml/font-{}.xml'.format(uid)
        self.html = html if html else self.get_test_html()

    def get_test_html(self, url=None):
        # 58简历页
        url = url if url else 'https://bj.58.com/qzyewu/pn2/?' \
                              'PGTID=0d303353-0000-1188-' \
                              '7c8a-829b2b71d0e8&ClickID=2'

        res = self.html_downloader.download(url)

        return res.text

    def save_font(self):
        """
        保存字体
        :return:
        """
        base64string = re.search('(?<=base64,).*?(?=\))', self.html).group()
        bin_data = base64.b64decode(base64string)
        with open(self.font_path, 'wb') as f:
            f.write(bin_data)

    def get_font_xml(self):
        """
        获取字体 xml
        :return:
        """
        font = TTFont(self.font_path)
        font.saveXML(self.xml_path)

    def analysis(self, is_clean=True, debug=False):
        """
        解析xml，获取web页面字符与文字key及文字value hash值的对应关系
        :param is_clean: 是否清楚字体文件及字体xml文件
        :param debug: 是否终端输出字体对照关系
        :return:
        """

        self.save_font()
        self.get_font_xml()
        with open(self.xml_path, 'rb') as xml:
            soups = self.html_parser.parser(xml.read())
            ttglyph_lst = soups.find('glyf').find_all('ttglyph')[1:]
            # 字体对应关系通过cmap进行处理
            map_lst = soups.find('cmap').find_all('map')
            map_dict = {}
            for map in map_lst:
                map_dict[map.get('name')] = map.get('code')
            # print(map_dict, len(map_dict))
            analysis_res = []
            for ttglyph in ttglyph_lst:
                analysis_dict = dict()
                analysis_dict['ttglyph_name'] = ttglyph.get('name')
                # analysis_dict['html_name'] = '&#x{};'.format(
                #     analysis_dict['ttglyph_name'][3:].lower())
                analysis_dict['html_name'] = '&#x{};'.format(
                    map_dict.get(ttglyph.get('name'))[2:])
                ttglyph_value = []
                contour_lst = ttglyph.find_all('contour')
                for contour in contour_lst:
                    pt_lst = contour.find_all('pt')
                    for pt in pt_lst:
                        ttglyph_value.append(pt.attrs)
                analysis_dict['font_hash'] = hashlib.sha1(
                    json.dumps(ttglyph_value, sort_keys=True).encode(
                        'utf-8')).hexdigest()
                analysis_res.append(analysis_dict)

                if debug:
                    print(analysis_dict['ttglyph_name'],  # 字体key
                          analysis_dict['html_name'],  # web页面显示值
                          analysis_dict['font_hash'])  # 字体内容哈希值

            if is_clean:
                os.remove(self.font_path)
                os.remove(self.xml_path)

            return analysis_res

    @staticmethod
    def get_real_font_mapping(analysis_res, font_mapping):
        real_font_mapping = dict()
        for item in analysis_res:
            real_font_mapping[item['html_name']] = font_mapping[
                item['font_hash']]
        return real_font_mapping

    @staticmethod
    def recover_html(html, real_font_mapping):
        for k, v in real_font_mapping.items():
            html = html.replace(k, v)
        return html


font_mapping = {
    'e8933be932e70f9afa7eaa9289db728428381314': '',
    '4fff0f1b24d0de9d52025a7a3a27da552ff57885': '技',
    '8cc5efb98e3e8f3cd7c5349a081a8a5c4584ff9b': '6',
    'd23c43c80a7498c5076156adc55826fbca139650': '经',
    'b7539565b06268d70a91cea2cd4178b4da798089': '王',
    '97817f695aacb359ffb5b8ca0717ca6344e84ccb': '应',
    'cc315ddd20b6361ef77cc82fbbc59f9ac40c3bd5': '专',
    '644caa33f64d9751aca184327eb27928b26775a7': '赵',
    'edc7496ac07d57b44a026de002422eec4f2ee1c4': '李',
    'cec031df25f4fc961f481e6029e9787fd0860d43': '以',
    'a07a8e4e1b273d55773605c33dbeb2b35ab0feff': '吴',
    '91106b5585628e13a9dd99c023436f0720204c33': '女',
    '50e496f505193e183f9114a98d1d84cd170d6841': '杨',
    '00acb9148038506dd725b383c8190a2db56ea763': '7',
    'ba582a83c79f815c38356c5d74b1695f08b62877': '5',
    'f60ba3e0090eb10b4a1815e0c054fa44b7303aa0': '张',
    '651cc305ff99243489da82fbfecf020446d7c342': 'B',
    'd5f3892cd55e9f22947ac2aba291329711533e81': '本',
    '22e30284b63bcf5f442f1ee12a1057d3601b11f3': '男',
    '3267ee0dd0cb0e11617788ae753cc369a5c4baae': '博',
    'fd05decfb4e1794a224a9129f6ef92993d325a81': '3',
    '14b002b6241afb1b968cf0b812854d9b0a212cdf': '无',
    'f3686b516fafbdcca4eeef2648e0951f3889de32': '9',
    'a03d1be9168f50f083686d63411c82cbea6db391': '生',
    '4f20412aff68f7f19790761a1cbcaeaa7db0c9e9': '验',
    '1354e12807c09a95d08ec737b6e94abf0396eeb0': '8',
    'a935b6d4b4c63c3c25314ed2e497570a697b88ff': '下',
    '0cb8891b4b93f0060b25a8a190b37ebf0fe4837e': '科',
    'c6cd476937866c3aa89dd73db226af18e9d4e3f4': '1',
    'f95ae3a1652a55f24f34d9c2f32eeac3f5230b2d': '4',
    '4083c6fe15c7f55624286691f09d18c8a54bc4f1': 'M',
    '9c22abfda3684ef55fb4d14c8abb2c8cc191cb74': '中',
    'a0d2037fded629d8ee3d3563c2974935e84f6e41': '硕',
    '40d32e734682aee7df1bacdae7db056019b7fdd6': '届',
    'a67d95e35ce49467e2b0ff549a173043460742d8': '2',
    '0889937589fdd669a172f4e65707938a3dc2242a': '刘',
    'f18e8a977aeb769874d0d8714c3153109658d1b5': '士',
    '0c61399311739c5a4d584447f4ef7d3b268cc51f': '陈',
    '7604e45e05cf4b3a98804474c15f2c4096fa410b': '高',
    'cd5372f2cd6103cd7895a4ca1847860da14a7e59': '大',
    'c901ab9eb8a4d5539778a3a34f17f0515cb2ef64': '0',
    '4b498c37e97d53a91d68bda4cfd4848adc977f06': 'E',
    '3486580a1462d28776136bf1f620b3dec79c982b': '周',
    '1ac40770b08fa65fac385230e8d9799257b47380': 'A',
    '9a379807d2bf090c94f1a03cf3fd18c6282882c1': '校',
    '77880914931fb6dda97269a9156404745f609d35': '黄'
}

# def main():
#     runner = FontAnalysis()
#     analysis_res = runner.analysis(debug=True)
#     real_font_mapping = runner.get_real_font_mapping(analysis_res, font_mapping)
#     print(real_font_mapping)
#
#     real_html = runner.recover_html(html=runner.html,
#                                     real_font_mapping=real_font_mapping)
#     print(real_html)
#
#
# if __name__ == '__main__':
#     main()
